@@ -49,29 +49,39 @@ class Helpers(object):
     # tuple: type((429)) is 'int', whereas type((429,)) is 'tuple'.
     # Borrowed from:
     # https://www.peterbe.com/plog/best-practice-with-retries-with-requests
+
+    retryclass = requests.packages.urllib3.util.Retry
+
     @staticmethod
-    def session_add_retry_handler(
-        retries=7, backoff_factor=1, status_forcelist=(429,), session=None
-    ):
-        session = session or requests.Session()
+    def session_add_retry_handler(session=None):
+        # urllib3 does not retry on POST by default, but we want to iff the
+        # return status is 429 rate limiting, on the assumption that this
+        # means the POST did not happen and is therefore safe to retry.
+        http_verbs = set(Helpers.retryclass.DEFAULT_ALLOWED_METHODS)
+        http_verbs.add('POST')
         retry = Helpers.create_retry_handler(
-            retries, backoff_factor, status_forcelist
+            retries=7,
+            backoff_factor=0.5,
+            status_forcelist=(429,),
+            allowed_methods=http_verbs
         )
 
         adapter = requests.adapters.HTTPAdapter(max_retries=retry)
 
+        session = session or requests.Session()
         session.mount(prefix='http://', adapter=adapter)
         session.mount(prefix='https://', adapter=adapter)
         return session
 
     @staticmethod
-    def create_retry_handler(retries, backoff_factor, status_forcelist):
+    def create_retry_handler(retries, backoff_factor, status_forcelist,
+                             allowed_methods):
         assert isinstance(retries, int)
-        assert retries >= 1
-        assert backoff_factor > 0
+        assert retries >= 0
+        assert backoff_factor >= 0
         assert isinstance(status_forcelist, tuple)
 
-        return requests.packages.urllib3.util.retry.Retry(
+        return Helpers.retryclass(
             total=retries,
             read=retries,
             connect=retries,
@@ -97,7 +107,8 @@ class PathTracker(object):
         new1.stack = self.stack
         return new1
 
-    def cd(self, path):
+    def cd(self, path=None):
+        path = str(path or '/')
         # no support for '..' right now, maybe in the future
         if path[0] == '/':
             self.prefix = path
@@ -106,7 +117,7 @@ class PathTracker(object):
         self.prefix = self.prefix.strip('/')
         return self.prefix
 
-    def pushd(self, path):
+    def pushd(self, path=None):
         self.stack.append(self.prefix)
         return self.cd(path)
 
@@ -114,8 +125,8 @@ class PathTracker(object):
         self.prefix = self.stack.pop()
         return self.prefix
 
-    def fullpath(self, suffix=''):
-        suffix = str(suffix)
+    def fullpath(self, suffix=None):
+        suffix = str(suffix or '')
         if len(suffix) > 0 and suffix[0] == '/':
             retval = suffix
         else:
@@ -152,11 +163,11 @@ class ApiObject(object):
         )
         return new1
 
-    def cd(self, path):
+    def cd(self, path=None):
         self.tracker.cd(path)
         return self.compute_url()
 
-    def pushd(self, path):
+    def pushd(self, path=None):
         self.tracker.pushd(path)
         return self.compute_url()
 
@@ -266,31 +277,31 @@ class ApiObject(object):
         except JSONDecodeError:
             return resp.text
 
-    def get(self, suffix='', headers=None):
+    def get(self, suffix=None, headers=None):
         url = self.compute_url(suffix)
         hdr = self.prep_headers(headers)
         resp = self.session.get(url, headers=hdr)
         return self.process_result(url, resp)
 
-    def post(self, suffix='', headers=None, data=None, json=None):
+    def post(self, suffix=None, headers=None, data=None, json=None):
         url = self.compute_url(suffix)
         hdr = self.prep_headers(headers)
         resp = self.session.post(url, headers=hdr, data=data, json=json)
         return self.process_result(url, resp)
 
-    def put(self, suffix='', headers=None, data=None, json=None):
+    def put(self, suffix=None, headers=None, data=None, json=None):
         url = self.compute_url(suffix)
         hdr = self.prep_headers(headers)
         resp = self.session.put(url, headers=hdr, data=data, json=json)
         return self.process_result(url, resp)
 
-    def delete(self, suffix='', headers=None, data=None, json=None):
+    def delete(self, suffix=None, headers=None, data=None, json=None):
         url = self.compute_url(suffix)
         hdr = self.prep_headers(headers)
         resp = self.session.delete(url, headers=hdr, data=data, json=json)
         return self.process_result(url, resp)
 
-    def patch(self, suffix='', headers=None, data=None, json=None):
+    def patch(self, suffix=None, headers=None, data=None, json=None):
         url = self.compute_url(suffix)
         hdr = self.prep_headers(headers)
         resp = self.session.patch(url, headers=hdr, data=data, json=json)
@@ -298,7 +309,7 @@ class ApiObject(object):
 
 
 class ApiWrapper(object):
-    def __init__(self, apiobject, suffix=''):
+    def __init__(self, apiobject, suffix=None):
         self.api = apiobject.clone()
         if suffix:
             self.api.chroot(suffix)
@@ -306,5 +317,17 @@ class ApiWrapper(object):
     def __str__(self):
         return '%s %s' % (str(type(self)), self.api)
 
-    def get(self, suffix='', headers=None):
-        return self.api.get(suffix, headers)
+    def get(self, suffix=None, headers=None):
+        return self.api.get(suffix, headers=headers)
+
+    def post(self, suffix=None, headers=None, data=None, json=None):
+        return self.api.post(suffix, headers=headers, data=data, json=json)
+
+    def put(self, suffix=None, headers=None, data=None, json=None):
+        return self.api.put(suffix, headers=headers, data=data, json=json)
+
+    def delete(self, suffix=None, headers=None, data=None, json=None):
+        return self.api.delete(suffix, headers=headers, data=data, json=json)
+
+    def patch(self, suffix=None, headers=None, data=None, json=None):
+        return self.api.patch(suffix, headers=headers, data=data, json=json)
